@@ -2,123 +2,101 @@ package Dancer2::Plugin::Auth::ActiveDirectory;
 
 =head1 NAME
 
-Dancer2::Plugin::Auth::ActiveDirectory - Authentication module for MS ActiveDirectory
+Dancer2::Plugin::Auth::ActiveDirectory - Dancer2 plugin for MS ActiveDirectory
 
 =head1 VERSION
 
-Version 0.02
+Version 0.03
 
 =cut
 
-our $VERSION = '0.02';
+our $VERSION = '0.03';
 
 use 5.10.0;
 use strict;
 use warnings FATAL => 'all';
 use Dancer2::Plugin;
-use Net::LDAP qw[];
-use Net::LDAP::Constant qw[LDAP_INVALID_CREDENTIALS];
+use Auth::ActiveDirectory;
 
 # -----------------------------------------------
 # Preloaded methods go here.
 # -----------------------------------------------
 # Encapsulated class data.
+my $_settings = undef;
 
-sub _authenticate {
-    my ( $dsl, $s_username, $s_auth_password ) = @_;
-    my $hr_stg = plugin_setting();
-    my $or_connection = Net::LDAP->new( $hr_stg->{host}, port => 389, timeout => 60 );
-    unless ( defined $or_connection ) {
-        my $host = $hr_stg->{host};
-        $dsl->error(qq/Failed to connect to '$host'. Reason: '$@'/);
-        return undef;
+=head1 PRIVATE METHODS
+
+=cut 
+
+{
+
+=head2 _load_settings
+ 
+Load plugin settings from environment.
+ 
+=cut
+
+    sub _load_settings() {
+        return $_settings if $_settings;
+        $_settings = plugin_setting;
+        return $_settings;
     }
-    my $s_principal = $hr_stg->{principal};
-    my $s_user      = sprintf( '%s@%s', $s_username, $s_principal );
-    my $message     = $or_connection->bind( $s_user, password => $s_auth_password );
-    return undef if ( $dsl->_v_is_error( $message, $s_user ) );
-    my $s_domain = $hr_stg->{domain};
-    my $result   = $or_connection->search(    # perform a search
-        base   => qq/dc=$s_principal,dc=$s_domain/,
-        filter => qq/(&(objectClass=person)(userPrincipalName=$s_user.$s_domain))/,
-    );
-    foreach ( $result->entries ) {
-        my $groups = [];
-        foreach my $group ( $_->get_value(q/memberOf/) ) {
-            push( @$groups, $1 ) if ( $group =~ m/^CN=(.*),OU=.*$/ );
-        }
-        return {
-            uid       => $s_username,
-            firstname => $_->get_value(q/givenName/),
-            surname   => $_->get_value(q/sn/),
-            groups    => $groups,
-            rights    => _rights_by_user( $hr_stg, $groups ),
-            user      => $s_user,
-        };
+
+=head2 _in_list
+ 
+Simple list in list comparision sub.
+ 
+=cut
+
+    sub _in_list {
+        !!grep {
+            exists { map { $_ => 1 } @{ $_[1] } }->{$_}
+        } @{ $_[0] };
     }
-    return undef;
-}
 
-sub _authenticate_config {
-    return plugin_setting();
-}
+=head2 _rights
+ 
+Get defined rights from environment
+ 
+=cut
 
-sub _has_right {
-    my ( $dsl, $o_session_user, $s_right_name ) = @_;
-    my $hr_rights  = plugin_setting()->{rights};
-    my $s_ad_group = $hr_rights->{$s_right_name};
-    return grep( /$s_ad_group/, @{ $o_session_user->{groups} } );
-}
+    sub _rights { _load_settings->{rights} || {} }
 
-sub _list_users {
-    my ( $dsl, $o_session_user, $search_string ) = @_;
-    my $hr_stg = plugin_setting();
-    my $or_connection = Net::LDAP->new( $hr_stg->{host}, port => 389, timeout => 60 );
-    unless ( defined $or_connection ) {
-        my $host = $hr_stg->{host};
-        $dsl->error(qq/Failed to connect to '$host'. Reason: '$@'/);
-        return undef;
+=head2 _connect_to_ad
+ 
+Creates a new L<Auth::ActiveDirectory> object including 
+ldap connection to AD server.
+ 
+=cut
+
+    sub _connect_to_ad {
+        return Auth::ActiveDirectory->new( @{ [ %{ _load_settings() } ] } );
     }
-    my $s_user = $o_session_user->{user};
-    my $message = $or_connection->bind( $s_user, password => $o_session_user->{password} );
 
-    return undef if ( $dsl->_v_is_error( $message, $s_user ) );
-    my $s_principal = $hr_stg->{principal};
-    my $s_domain    = $hr_stg->{domain};
-    my $result      = $or_connection->search(
-        base   => qq/dc=$s_principal,dc=$s_domain/,
-        filter => qq/(&(objectClass=person)(name=$search_string*))/,
-    );
-    my $return_names = [];
-    push( @$return_names, { name => $_->get_value(q/name/), uid => $_->get_value(q/sAMAccountName/), } ) foreach ( $result->entries );
-    return $return_names;
-}
+=head2 _rights_by_user_groups
+ 
+Returns a hashref with the possible rights.
+Based on the AD groups where the user is included.
+ 
+=cut
 
-sub _v_is_error {
-    my ( $dsl, $message, $s_user ) = @_;
-    if ( $message->is_error ) {
-        my $error = $message->error;
-        my $level = $message->code == LDAP_INVALID_CREDENTIALS ? 'debug' : 'error';
-        $dsl->error(qq/Failed to authenticate user '$s_user'. Reason: '$error'/);
-        return 1;
+    sub _rights_by_user {
+        my ( $dsl, $user ) = @_;
+        return _rights_by_user_groups( $dsl, $user->{groups} );
     }
-    return 0;
-}
 
-sub _rights {
-    return plugin_setting()->{rights};
-}
+=head2 _rights_by_user_groups
 
-sub _rights_by_user {
-    my ( $hr_stg, $a_user_groups ) = @_;
-    my $hr_rights = $hr_stg->{rights};
-    return unless $hr_rights;
-    my $ret_rights = {};
-    foreach ( keys %$hr_rights ) {
-        my $s_ad_group = $hr_rights->{$_};
-        $ret_rights->{$_} = 1 if ( grep( /$s_ad_group/, @{$a_user_groups} ) );
+Returns a hashref with the possible rights.
+Based on the AD groups where the user is included.
+
+=cut
+
+    sub _rights_by_user_groups {
+        my ( $dsl, $user_groups ) = @_;
+        my $rights = _rights($dsl);
+        return { map { _in_list( $user_groups, ( ref $rights->{$_} ne 'ARRAY' ) ? [ $rights->{$_} ] : $rights->{$_} ) ? ( $_ => 1 ) : () } keys %{$rights} };
     }
-    return $ret_rights;
 }
 
 =head1 SYNOPSIS
@@ -155,7 +133,22 @@ Basicaly the subroutine for authentication in the ActiveDirectory
 
 =cut
 
-register authenticate => \&_authenticate;
+register authenticate => sub {
+    my ( $dsl, $name, $pass ) = @_;
+    my $user = _connect_to_ad($dsl)->authenticate( $name, $pass );
+    return $user if $user->{error};
+    my $user_groups = [ map { $_->name } @{ $user->groups } ];
+    return {
+        uid          => $user->uid,
+        firstname    => $user->firstname,
+        surname      => $user->surname,
+        mail         => $user->mail,
+        display_name => $user->display_name,
+        user         => $user->user,
+        groups       => $user_groups,
+        rights       => _rights_by_user_groups( $dsl, $user_groups ),
+    };
+};
 
 =head2 authenticate_config
 
@@ -163,7 +156,7 @@ Subroutine to get configuration for ActiveDirectory
 
 =cut
 
-register authenticate_config => \&_authenticate_config;
+register authenticate_config => \&_load_settings;
 
 =head2 has_right
 
@@ -171,13 +164,15 @@ Check if loged in user has one of the configured rights
 
 =cut
 
-register has_right => \&_has_right;
+register has_right => sub {
+    return $_[1]->{rights}->{ $_[2] } ? 1:0;
+};
 
 =head2 list_users
 
 =cut
 
-register list_users => \&_list_users;
+register list_users => sub { _connect_to_ad(shift)->list_users(@_) };
 
 =head2 rights
 
@@ -187,6 +182,14 @@ Subroutine to get configurated rights
 
 register rights => \&_rights;
 
+=head2 rights_by_user
+
+Subroutine to get configurated rights
+
+=cut
+
+register rights_by_user => sub { _rights_by_user( $_[0], $_[1] ) };
+
 =head2 for_versions
 
 =cut
@@ -210,10 +213,9 @@ automatically be notified of progress on your bug as I make changes.
 
 =head1 MOTIVATION
 
-If you have a run in programming you don't always notice all packages in this moment.
-And later when someone will know which packages are used, it's not neccessary to look at all of the packages.
-
-Usefull for the Makefile.PL or Build.PL.
+I started to write this module for an internal application based on Dancer2.
+The authentication should be from our AD servers ( don't ask... ),
+the result ist this Module and L<Auth::ActiveDirectory>.
 
 
 =head1 SUPPORT
